from blueness import module

from blue_objects.metadata import post_to_object

from blue_geo import NAME
from blue_geo.logger import logger

NAME = module.name(__file__, NAME)


def prep_dataset(
    module_name: str,
    query_object_name: str,
) -> bool:
    logger.info(
        "{}.prep_dataset: {} @ {}...".format(
            NAME,
            module_name,
            query_object_name,
        )
    )

    return all(
        post_to_object(
            query_object_name,
            keyword,
            value,
        )
        for keyword, value in {
            "kind": "distributed",
            "module_name": module_name,
            "source": "catalog_query",
        }.items()
    )



from typing import Dict
import numpy as np
import rasterio
from rasterio.features import rasterize
from tqdm import tqdm

from blueness import module
from blue_objects import objects, file
from blue_objects.metadata import post_to_object

from blue_geo import NAME
from blue_geo.file.load import load_geodataframe
from blue_geo.catalog import get_datacube
from blue_geo.catalog.generic.generic.scope import DatacubeScope
from blue_geo.logger import logger

NAME = module.name(__file__, NAME)


def rasterize_the_label(
    datacube_id: str,
    verbose: bool = False,
) -> bool:
    logger.info(f"{NAME}.rasterize_the_label: {datacube_id}")

    datacube = get_datacube(datacube_id)

    list_of_files = datacube.list_of_files(scope=DatacubeScope("rgb"))
    if not list_of_files:
        logger.error("cannot find a reference file.")
        return False

    reference_filename = list_of_files[0]
    logger.info(f"reference: {reference_filename}")

    label_filename = objects.path_of(
        filename="label.shp",
        object_name=datacube_id,
    )
    success, label_polygons = load_geodataframe(label_filename)
    if not success:
        return success
    logger.info(
        "label: {} - {} polygon(s)".format(
            file.name_and_extension(label_filename),
            len(label_polygons),
        )
    )

    success, label_template_polygons = load_geodataframe(
        objects.path_of(
            filename="template/label.shp",
            object_name=datacube_id,
        )
    )
    if not success:
        return success
    list_of_classes = sorted(list(set(label_template_polygons["class"].tolist())))
    if "background" in list_of_classes:
        list_of_classes = ["background"] + [
            class_name for class_name in list_of_classes if class_name != "background"
        ]
    logger.info(
        "{} class(es): {}".format(
            len(list_of_classes),
            ", ".join(list_of_classes),
        )
    )

    class_pixel_count: Dict[str, int] = {
        class_name: 0 for class_name in list_of_classes
    }
    with rasterio.open(
        objects.path_of(
            filename=reference_filename,
            object_name=datacube_id,
        )
    ) as src:
        label_image = np.zeros(src.shape, dtype=np.uint8)

        transform = src.transform
        meta = src.meta
        meta["count"] = 1

        label_polygons = label_polygons.to_crs(src.crs)

        for _, row in tqdm(label_polygons.iterrows()):
            class_name = row["class"]

            class_index = list_of_classes.index(class_name)

            polygon_mask = rasterize(
                [(row.geometry, class_index)],
                out_shape=src.shape,
                fill=0,
                transform=transform,
                all_touched=True,
                dtype=np.uint8,
            )

            pixel_count = int(np.sum(polygon_mask != 0))
            class_pixel_count[class_name] += pixel_count
            if verbose:
                logger.info(
                    "#{} {}: {:,} pixel(s)".format(
                        class_index, class_name, pixel_count
                    ),
                )

            label_image = np.maximum(label_image, polygon_mask)

    for class_name in list_of_classes:
        logger.info(f"{class_name}: {class_pixel_count[class_name]:,} pixel(s)")

    with rasterio.open(
        objects.path_of(
            filename=file.add_suffix(reference_filename, "label"),
            object_name=datacube_id,
        ),
        "w",
        **meta,
    ) as dst:
        dst.write(label_image, 1)

    return post_to_object(
        datacube_id,
        "rasterize",
        {
            "counts": class_pixel_count,
            "label_count": len(label_polygons),
            "label_filename": file.name_and_extension(label_filename),
            "list_of_classes": list_of_classes,
            "reference_filename": reference_filename,
        },
    )



from typing import Tuple, Any, Any, Dict
import numpy as np
import geopandas as gpd
import rasterio
import geojson

from blueness import module
from blue_options import string
from blue_options.logger import crash_report

from blue_geo import NAME
from blue_geo.logger import logger

NAME = module.name(__file__, NAME)


def load_geodataframe(
    filename: str,
    ignore_error: bool = False,
    log: bool = False,
) -> Tuple[bool, Any]:
    success = False
    gdf = None

    try:
        gdf = gpd.read_file(filename)
        success = True
    except:
        if not ignore_error:
            crash_report(f"{NAME}: load_geodataframe({filename}): failed.")

    if success and log:
        logger.info(
            "loaded {:,} rows: {} from {}".format(
                len(gdf),
                ", ".join(gdf.columns),
                filename,
            )
        )

    return success, gdf


def load_geoimage(
    filename: str,
    ignore_error: bool = False,
    log: bool = False,
) -> Tuple[bool, np.ndarray, Dict[str, Any]]:
    success = False
    image = np.empty((0,))
    pixel_size = -1.0
    crs = "unknown"

    try:
        with rasterio.open(filename) as src:
            image = src.read()

            pixel_size = src.res

            crs = src.crs

        success = True
    except:
        if not ignore_error:
            crash_report(f"{NAME}: load_geoimage({filename}): failed.")

    if success and log:
        logger.info(
            "loaded {} @ {} m : {} from {}".format(
                string.pretty_shape_of_matrix(image),
                pixel_size,
                crs,
                filename,
            )
        )

    return (
        success,
        image,
        {
            "pixel_size": pixel_size,
            "crs": crs,
        },
    )


# https://stackoverflow.com/a/47792385/17619982
def load_geojson(
    filename,
    ignore_error=False,
) -> Tuple[bool, Any]:
    success = False
    data = {}

    try:
        with open(filename, "r") as fh:
            data = geojson.load(fh)

        success = True
    except:
        if not ignore_error:
            crash_report(f"{NAME}: load_geojson({filename}): failed.")

    return success, data



import os
from blue_options.env import load_config, load_env, get_env

load_env(__name__)
load_config(__name__)


# secrets
FIRMS_MAP_KEY = get_env("FIRMS_MAP_KEY")

COPERNICUS_AWS_ACCESS_KEY_ID = get_env("COPERNICUS_AWS_ACCESS_KEY_ID")

COPERNICUS_AWS_SECRET_ACCESS_KEY = get_env("COPERNICUS_AWS_SECRET_ACCESS_KEY")

SKYFOX_ACCESS_TOKEN_URL = get_env("SKYFOX_ACCESS_TOKEN_URL")
SKYFOX_CLIENT_ID = get_env("SKYFOX_CLIENT_ID")
SKYFOX_CLIENT_SECRET = get_env("SKYFOX_CLIENT_SECRET")

# config

BLUE_GEO_DISABLED_CATALOGS = get_env("BLUE_GEO_DISABLED_CATALOGS")

BLUE_GEO_WATCH_TARGET_LIST = get_env("BLUE_GEO_WATCH_TARGET_LIST")

BLUE_GEO_SKYFOXCATALOG_API_URL = get_env("BLUE_GEO_SKYFOXCATALOG_API_URL")

BLUE_GEO_SKYFOXCATALOG_API_GET_TOKEN = get_env("BLUE_GEO_SKYFOXCATALOG_API_GET_TOKEN")

# QGIS templates
BLUE_GEO_QGIS_TEMPLATE_FIRMS_AREA = get_env("BLUE_GEO_QGIS_TEMPLATE_FIRMS_AREA")

BLUE_GEO_QGIS_TEMPLATE_DATACUBE_SKYFOX_VENUS = get_env(
    "BLUE_GEO_QGIS_TEMPLATE_DATACUBE_SKYFOX_VENUS"
)

BLUE_GEO_QGIS_TEMPLATE_UKRAINE_TIMEMAP = get_env(
    "BLUE_GEO_QGIS_TEMPLATE_UKRAINE_TIMEMAP"
)

BLUE_GEO_QGIS_TEMPLATE_WATCH = get_env("BLUE_GEO_QGIS_TEMPLATE_WATCH")


# test datacube-ids
BLUE_GEO_TEST_DATACUBE_COPERNICUS_SENTINEL_2 = get_env(
    "BLUE_GEO_TEST_DATACUBE_COPERNICUS_SENTINEL_2"
)
BLUE_GEO_TEST_DATACUBE_COPERNICUS_SENTINEL_2_CHILCOTIN = get_env(
    "BLUE_GEO_TEST_DATACUBE_COPERNICUS_SENTINEL_2_CHILCOTIN"
)

BLUE_GEO_TEST_DATACUBE_CROP_CTULINE = get_env("BLUE_GEO_TEST_DATACUBE_CROP_CTULINE")

BLUE_GEO_TEST_DATACUBE_EARTHSEARCH_SENTINEL2_L1C = get_env(
    "BLUE_GEO_TEST_DATACUBE_EARTHSEARCH_SENTINEL2_L1C"
)

BLUE_GEO_TEST_DATACUBE_FIRMS_AREA = get_env("BLUE_GEO_TEST_DATACUBE_FIRMS_AREA")

BLUE_GEO_TEST_DATACUBE_GENERIC_GENERIC = get_env(
    "BLUE_GEO_TEST_DATACUBE_GENERIC_GENERIC"
)

BLUE_GEO_TEST_DATACUBE_SKYFOX_VENUS = get_env("BLUE_GEO_TEST_DATACUBE_SKYFOX_VENUS")

BLUE_GEO_TEST_DATACUBE_UKRAINE_TIMEMAP = get_env(
    "BLUE_GEO_TEST_DATACUBE_UKRAINE_TIMEMAP"
)

BLUE_GEO_WATCH_ALGO_DIFF_MAP_DYNAMIC_RANGE = get_env(
    "BLUE_GEO_WATCH_ALGO_DIFF_MAP_DYNAMIC_RANGE"
)

BLUE_GEO_TEST_DATACUBE_MAXAR_OPEN_DATA = get_env(
    "BLUE_GEO_TEST_DATACUBE_MAXAR_OPEN_DATA"
)

BLUE_GEO_QGIS_TEMPLATE_MAXAR_OPEN_DATA = get_env(
    "BLUE_GEO_QGIS_TEMPLATE_MAXAR_OPEN_DATA"
)

MAXAR_OPEN_DATA_CLIENT_QUERY_RADIUS = get_env(
    "MAXAR_OPEN_DATA_CLIENT_QUERY_RADIUS", 0.01
)

MAXAR_OPEN_DATA_CLIENT_CACHE_ITEMS = get_env("MAXAR_OPEN_DATA_CLIENT_CACHE_ITEMS", True)

BLUE_GEO_PALISADES_TEST_DATACUBE = get_env("BLUE_GEO_PALISADES_TEST_DATACUBE")

BLUE_GEO_TEST_QUERY_OBJECT_PALISADES_MAXAR_TEST = get_env(
    "BLUE_GEO_TEST_QUERY_OBJECT_PALISADES_MAXAR_TEST"
)


BLUE_GEO_FILE_LOAD_GEOIMAGE_TEST_OBJECT = get_env(
    "BLUE_GEO_FILE_LOAD_GEOIMAGE_TEST_OBJECT"
)
BLUE_GEO_FILE_LOAD_GEOIMAGE_TEST_FILENAME = get_env(
    "BLUE_GEO_FILE_LOAD_GEOIMAGE_TEST_FILENAME"
)

BLUE_GEO_TEST_OBJECT = get_env("BLUE_GEO_TEST_OBJECT")



import math
from qgis.core import *
from qgis.gui import *

# version 2.3.1


@qgsfunction(args="auto", group="Custom", referenced_columns=[])
def palisades_label(row, feature, parent):
    """
    Produce label text for a palisades mapid.

    palisades_label(row)
    """

    area = row["area"]
    damage = row["damage"]

    return "{:,.0f} sq. m{}".format(
        area,
        " - {:.1f} %".format(100 * damage) if damage > 0 else "",
    )



import numpy as np
import rasterio
import geopandas as gpd

from blueness import module
from blue_options import string
from blue_objects.file.save import prepare_for_saving, finish_saving

from blue_geo import NAME

NAME = module.name(__file__, NAME)


def save_geodataframe(
    filename: str,
    gdf: gpd.GeoDataFrame,
    log: bool = False,
):
    if not prepare_for_saving(filename):
        return False

    success = True
    try:
        gdf.to_file(filename)
    except:
        success = False

    return finish_saving(
        success,
        "{}.save_geodataframe: {} -> {}".format(
            NAME,
            f"{len(gdf)} row(s)",
            filename,
        ),
        log,
    )


def save_geojson(
    filename: str,
    gdf: gpd.GeoDataFrame,
    log: bool = False,
):
    if not prepare_for_saving(filename):
        return False

    success = True
    try:
        gdf.to_file(filename, driver="GeoJSON")
    except:
        success = False

    return finish_saving(
        success,
        "{}.save_geojson: {:,}X[{}] row(s) -> {}".format(
            NAME,
            len(gdf),
            ",".join(list(gdf.columns)),
            filename,
        ),
        log,
    )


def save_geoimage(
    filename: str,
    image: np.ndarray,
    template_filename: str,
    log: bool = False,
    dtype: str = rasterio.float32,
):
    if not prepare_for_saving(filename):
        return False

    success = True
    try:
        with rasterio.open(template_filename) as src:
            profile = src.profile

            profile.update(
                dtype=dtype,
                count=image.shape[2],
                nodata=0,
            )
            with rasterio.open(filename, "w", **profile) as dst:
                dst.write(image)
    except:
        success = False

    return finish_saving(
        success,
        "{}.save_geoimage: {} -> {}".format(
            NAME,
            string.pretty_shape_of_matrix(image),
            filename,
        ),
        log,
    )



import glob

if not QGIS_is_live:
    from ..logger import Q_log
    from ..file import Q_get_file_name, Q_file_exists
    from ..application import BLUE_GEO_QGIS_APPLICATION
    from ..QGIS import QGIS
    from ..objects import Q_file_path_in_object
    from ..project import Q_project
    from ..logger import Q_log_error, Q_verbose
    from ..seed import Q_seed


class BLUE_GEO_QGIS_APPLICATION_PALISADES(BLUE_GEO_QGIS_APPLICATION):
    def __init__(self):
        super().__init__("palisades", "🧑🏽‍🚒")

    def help_(self):
        Q_log(
            "palisades.load",
            "load the layers of the palisades object.",
            icon=self.icon,
        )

    @property
    def load(self):
        metadata = Q_project.metadata
        if Q_verbose:
            self.log(f"metadata: {metadata}")

        if "predict" not in metadata:
            Q_log_error("predict not found in metadata.")
            return

        reference_filename = metadata["predict"]["reference_filename"]
        datacube_id = metadata["predict"]["datacube_id"]
        filename = Q_file_path_in_object(
            filename=reference_filename,
            object_name=datacube_id,
        )
        if not Q_file_exists(filename):
            Q_seed(
                [
                    "blue_geo_datacube_ingest",
                    "scope=rgb",
                    datacube_id,
                ]
            )
            return

        if not Q_project.add_layer(
            filename=filename,
            layer_name=f"input-{reference_filename}",
        ):
            return

        output_filename = metadata["predict"]["output_filename"]
        if not Q_project.add_layer(
            filename=Q_file_path_in_object(
                filename=output_filename,
                object_name=Q_project.name,
            ),
            layer_name=f"predict-{output_filename}",
            template_name="prediction-template",
        ):
            return

        filename = Q_file_path_in_object(
            filename="analysis.gpkg",
            object_name=Q_project.name,
        )
        if not Q_file_exists(filename):
            filename = ""

            building_footprint_filename_list = glob.glob(
                Q_file_path_in_object(
                    filename="*.gpkg",
                    object_name=Q_project.name,
                )
            )
            if building_footprint_filename_list:
                filename = building_footprint_filename_list[0]

        if filename and not Q_project.add_layer(
            filename=filename,
            layer_name=Q_get_file_name(filename),
            template_name=f"{Q_get_file_name(filename)}-template",
        ):
            return


palisades = BLUE_GEO_QGIS_APPLICATION_PALISADES()
QGIS.add_app(palisades)



import os
from typing import List, Union
import time
import random

from qgis.core import *
from qgis.gui import *


@qgsfunction(args="auto", group="Custom", referenced_columns=[])
def palisades_display(layer_filename, row, feature, parent):
    """
    Produce display text for a palisades mapid.

    palisades_display(
        layer_property(@layer,'path'),
        attributes($currentfeature)
    )
    """
    version = "5.53.1"

    def save_text(
        filename: str,
        text: List[str],
    ):
        with open(filename, "w") as fp:
            fp.writelines([string + "\n" for string in text])

    def seed(
        command: Union[str, List[str]],
        submit: bool = True,
    ):
        if isinstance(command, list):
            command = " ".join(command)

        if submit:
            command_name = "{}-{:05d}".format(
                time.strftime(
                    "QGIS-command-%Y-%m-%d-%H-%M-%S", time.localtime(time.time())
                ),
                random.randrange(100000),
            )

            blue_geo_QGIS_path_server = os.path.join(
                os.getenv("HOME", ""),
                "Downloads/QGIS/server",
            )

            save_text(
                filename=os.path.join(
                    blue_geo_QGIS_path_server,
                    f"{command_name}.command",
                ),
                text=[command],
            )

        return [
            '<label for="seed">🌱</label>',
            f'<input type="text" value="{command}" id="seed" style="background-color: white; color: black; width: 100%;">',
        ]

    area = row["area"]
    damage = float(row["damage"])

    layer_path, layer_filename = os.path.split(layer_filename)
    object_name = layer_path.split(os.sep)[-1]
    object_root = os.sep.join(layer_path.split(os.sep)[:-1])
    is_analytics = "analytics.geojson" in layer_filename

    thumbnail_object_name = row["thumbnail_object"] if is_analytics else object_name

    thumbnail_filename = row["thumbnail"]
    thumbnail_full_filename = os.path.join(
        object_root,
        thumbnail_object_name,
        thumbnail_filename,
    )

    return "\n".join(
        [
            '<p style="color: {};">{}</p>'.format(
                "green" if damage == 0 else "yellow" if damage < 0.1 else "red",
                " | ".join(
                    [
                        "area: {:,.0f} sq. m".format(area),
                        "damage: {:.1f}%{}".format(
                            100 * damage,
                            (
                                " +- {:.1f}%".format(100 * row["damage_std"])
                                if is_analytics
                                else ""
                            ),
                        ),
                    ]
                    + (
                        ["{} observation(s)".format(row["observation_count"])]
                        if is_analytics
                        else []
                    )
                ),
            ),
            "<hr/>",
        ]
        + (
            ['<img src="file://{}" width=500 >'.format(thumbnail_full_filename)]
            if os.path.exists(thumbnail_full_filename)
            else seed(
                [
                    "abcli",
                    "download",
                    f"filename={thumbnail_filename}",
                    thumbnail_object_name,
                ]
            )
        )
        + ["<hr/>"]
        + (
            seed(
                [
                    "palisades",
                    "analytics",
                    "ingest_building",
                    "building={},~download".format(row["building_id"]),
                    object_name,
                    ";",
                    "abcli",
                    "open",
                    "QGIS",
                    object_name,
                ],
                submit=False,
            )
            if is_analytics
            else []
        )
        + [
            "<hr/>",
            '<p style="color: white; width: 500px">{}</p>'.format(
                " | ".join(
                    (
                        [
                            "analytics",
                            row["building_id"],
                        ]
                        if is_analytics
                        else []
                    )
                    + [
                        object_name,
                        f"template-{version}",
                    ]
                )
            ),
        ],
    )
